'use strict';

var tslib_es6 = require('../../../external/tslib/tslib.es6.js');
var clientSsm = require('@aws-sdk/client-ssm');
var common = require('@nestjs/common');
var constant = require('../../../shared/constant/config/constant.js');

exports.ParameterStoreService = class ParameterStoreService {
    client;
    constructor(client) {
        this.client = client;
    }
    /**
     * Retrieves parameters from AWS Parameter Store by the specified path.
     * @param {string} path - The path to the parameters in AWS Parameter Store.
     * @param {boolean} shouldDecryptParameters - Whether to decrypt the parameter values. Default is false.
     * @param {boolean} shouldUseRecursiveLoading - Whether to recursively load parameters under the specified path. Default is false.
     * @param {boolean} isVerbose - Whether to log verbose messages. Default is false.
     * @returns {Promise<Array<Parameter>>} - A promise that resolves to an array of Parameter objects.
     */
    async getParametersByPath(path, shouldDecryptParameters = false, shouldUseRecursiveLoading = false, isVerbose = false) {
        if (isVerbose)
            common.Logger.verbose(`Getting parameters from path: ${path}`, "ParameterStoreConfig");
        let allParameters = [];
        let nextParametersToken;
        do {
            const getParameters = new clientSsm.GetParametersByPathCommand({
                NextToken: nextParametersToken,
                Path: path,
                // eslint-disable-next-line @elsikora/typescript/naming-convention
                Recursive: shouldUseRecursiveLoading,
                // eslint-disable-next-line @elsikora/typescript/naming-convention
                WithDecryption: shouldDecryptParameters,
            });
            const { NextToken, Parameters = [] } = await this.client.send(getParameters);
            allParameters = [...allParameters, ...Parameters];
            nextParametersToken = NextToken;
        } while (nextParametersToken);
        if (isVerbose)
            common.Logger.verbose(`Found ${String(allParameters.length)} parameters`, "ParameterStoreConfig");
        return allParameters;
    }
};
exports.ParameterStoreService = tslib_es6.__decorate([
    common.Injectable(),
    tslib_es6.__param(0, common.Inject(constant.PARAMETER_STORE_SSM_CLIENT)),
    tslib_es6.__metadata("design:paramtypes", [clientSsm.SSMClient])
], exports.ParameterStoreService);
//# sourceMappingURL=parameter-store.service.js.map
