import { __decorate, __param, __metadata } from '../../../external/tslib/tslib.es6.js';
import { GetParametersByPathCommand, SSMClient } from '@aws-sdk/client-ssm';
import { Injectable, Inject, Logger } from '@nestjs/common';
import { PARAMETER_STORE_SSM_CLIENT } from '../../../shared/constant/config/constant.js';

let ParameterStoreService = class ParameterStoreService {
    client;
    constructor(client) {
        this.client = client;
    }
    /**
     * Retrieves parameters from AWS Parameter Store by the specified path.
     * @param {string} path - The path to the parameters in AWS Parameter Store.
     * @param {boolean} shouldDecryptParameters - Whether to decrypt the parameter values. Default is false.
     * @param {boolean} shouldUseRecursiveLoading - Whether to recursively load parameters under the specified path. Default is false.
     * @param {boolean} isVerbose - Whether to log verbose messages. Default is false.
     * @returns {Promise<Array<Parameter>>} - A promise that resolves to an array of Parameter objects.
     */
    async getParametersByPath(path, shouldDecryptParameters = false, shouldUseRecursiveLoading = false, isVerbose = false) {
        if (isVerbose)
            Logger.verbose(`Getting parameters from path: ${path}`, "ParameterStoreConfig");
        let allParameters = [];
        let nextParametersToken;
        do {
            const getParameters = new GetParametersByPathCommand({
                NextToken: nextParametersToken,
                Path: path,
                // eslint-disable-next-line @elsikora/typescript/naming-convention
                Recursive: shouldUseRecursiveLoading,
                // eslint-disable-next-line @elsikora/typescript/naming-convention
                WithDecryption: shouldDecryptParameters,
            });
            const { NextToken, Parameters = [] } = await this.client.send(getParameters);
            allParameters = [...allParameters, ...Parameters];
            nextParametersToken = NextToken;
        } while (nextParametersToken);
        if (isVerbose)
            Logger.verbose(`Found ${String(allParameters.length)} parameters`, "ParameterStoreConfig");
        return allParameters;
    }
};
ParameterStoreService = __decorate([
    Injectable(),
    __param(0, Inject(PARAMETER_STORE_SSM_CLIENT)),
    __metadata("design:paramtypes", [SSMClient])
], ParameterStoreService);

export { ParameterStoreService };
//# sourceMappingURL=parameter-store.service.js.map
